==> scilab-2025.1.0.tar.bz2: scilab-2025.1.0/scilab/modules/scicos/sci_gateway/cpp/sci_scicosDiagramToScilab.cpp <==
/*
 * Scilab ( https://www.scilab.org/ ) - This file is part of Scilab
 * Copyright (C) Scilab Enterprises - 2016-2016 - Clément DAVID
 *
 * Copyright (C) 2012 - 2016 - Scilab Enterprises
 *
 * This file is hereby licensed under the terms of the GNU GPL v2.0,
 * pursuant to article 5.3.4 of the CeCILL v.2.1.
 * This file was originally licensed under the terms of the CeCILL v2.1,
 * and continues to be available under such terms.
 * For more information, see the COPYING file which you should have received
 * along with this program.
 *
 */
/*--------------------------------------------------------------------------*/
#include "Controller.hxx"
#include "view_scilab/Adapters.hxx"
#include "LoggerView.hxx"
#include "controller_helpers.hxx"
#include "XMIResource.hxx"
#include "SSPResource.hxx"

#include "types.hxx"
#include "function.hxx"
#include "string.hxx"
#include "user.hxx"
#include "int.hxx"

#include "gw_scicos.hxx"

extern "C"
{
#include "sci_malloc.h"
#include "localization.h"
#include "getFullFilename.h"
#include "Sciwarning.h"
#include "Scierror.h"
}
/*--------------------------------------------------------------------------*/
using namespace org_scilab_modules_scicos;
/*--------------------------------------------------------------------------*/
static char funname[] = "scicosDiagramToScilab";
static types::InternalType* importFile(char const* file);
static bool exportFile(int index, char const* file, types::InternalType* uid);
/*--------------------------------------------------------------------------*/
types::Function::ReturnValue sci_scicosDiagramToScilab(types::typed_list &in, int _iRetCount, types::typed_list &out)
{
    /*
     * to import some files
     * scs_m = scicosDiagramToScilab("/path/to/a/file")
     * [scs_m1, scs_m2] = scicosDiagramToScilab(["/path/to/a/file", "/path/to/files"])
     *
     * to export a file
     * scicosDiagramToScilab("/path/to/a/file", scs_m)
     * scicosDiagramToScilab(["/path/to/a/file", "/path/to/files"], scs_m1, scs_m2)
     */

    _iRetCount = std::max(1, _iRetCount);
    if (in.size() < 1)
    {
        Scierror(77, _("%s: Wrong number of input arguments: at least %d expected.\n"), funname, 1);
        return types::Function::Error;
    }

    if (!in[0]->isString())
    {
        Scierror(77, _("%s: Wrong type for input argument #%d: string expected.\n"), funname, 1);
        return types::Function::Error;
    }

    types::String* files = in[0]->getAs<types::String>();
    if (in.size() == 1 && files->getSize() == _iRetCount)
    {
        // the user asked to import
        out.resize(_iRetCount);
        for (int i = 0; i < _iRetCount; i++)
        {
            wchar_t* fullName = getFullFilenameW(files->get(i));
            char* f = wide_string_to_UTF8(fullName);
            FREE(fullName);
            out[i] = importFile(f);
            FREE(f);
            if (out[i] == nullptr)
            {
                return types::Function::Error;
            }
        }
    }
    else if (in.size() == 1 + static_cast<size_t>(files->getSize()) && _iRetCount <= 1)
    {
        // the user asked to export
        for (int i = 0; i < _iRetCount; i++)
        {
            if (!in[1 + i]->isUserType())
            {
                Scierror(77, _("%s: Wrong type for input argument #%d: ""%s"" expected.\n"), funname, i + 2, "diagram");
                return types::Function::Error;
            }
        }
        for (int i = 0; i < _iRetCount; i++)
        {
            wchar_t* fullName = getFullFilenameW(files->get(i));
            char* f = wide_string_to_UTF8(fullName);
            FREE(fullName);
            bool success = exportFile(1 + i, f, in[1 + i]);
            FREE(f);
            if (!success)
            {
                return types::Function::Error;
            }
        }
    }
    else
    {
        // report a comprehensible error message
        if (in.size() == 1)
        {
            Scierror(77, _("%s: Wrong number of output arguments: %d expected.\n"), funname, files->getSize());
        }
        else if (in.size() == 1 + static_cast<size_t>(files->getSize()))
        {
            Scierror(77, _("%s: Wrong number of output arguments: %d expected.\n"), funname, 0);
        }
        else
        {
            Scierror(77, _("%s: Wrong number of input arguments: %d expected.\n"), funname, 1);
        }
        return types::Function::Error;
    }

    return types::Function::OK;
}
/*--------------------------------------------------------------------------*/
static types::InternalType* importFile(char const* file)
{
    // create a diagram
    org_scilab_modules_scicos::Controller controller;
    ScicosID uid = controller.createObject(DIAGRAM);
    
    auto logger = get_or_allocate_logger();
    logger->log(LOG_DEBUG, [&](std::stringstream& msg){
        msg << "Importing file " << file << "\n";
        // msg << "Reset logger to " << uid << "\n";
        // logger->setLastObject(uid);
    });

    size_t fLen = strlen(file);
    int ret = -1;
    if (fLen < 4)
    {
        Scierror(999, _("%s: Unable to load \"%s\" .\n"), funname, file);
        controller.deleteObject(uid);
        return nullptr;
    }
    if (strcmp(&file[fLen - 4], ".ssp") == 0)
    {
        ret = SSPResource(uid).load(file);
    }
    else if (strcmp(&file[fLen - 4], ".xmi") == 0)
    {
        ret = XMIResource(uid).load(file);
    }
    else
    {
        Scierror(999, _("%s: Unable to detect \"%s\" format.\n"), funname, file);
        controller.deleteObject(uid);
        return nullptr;
    }

    if (ret < 0)
    {
        Scierror(999, _("%s: Unable to load file \"%s\" .\n"), funname, file);
        controller.deleteObject(uid);
        return nullptr;
    }

    types::InternalType* pIT = view_scilab::Adapters::instance().allocate_view(uid, DIAGRAM);
    return pIT;
}
/*--------------------------------------------------------------------------*/
static bool exportFile(int index, char const* file, types::InternalType* type)
{
    // check that the passed argument is a diagram
    const model::BaseObject* o = view_scilab::Adapters::instance().descriptor(type);
    if (o == nullptr || o->kind() != DIAGRAM)
    {
        Scierror(77, _("%s: Wrong type for input argument #%d: ""%s"" expected.\n"), funname, index, "diagram");
        return false;
    }

    size_t fLen = strlen(file);
    if (fLen < 4)
    {
        Scierror(999, _("%s: Unable to save \"%s\" .\n"), funname, file);
        return false;
    }
    
    const char* file_ext = &file[fLen - 4];
    if (strcmp(file_ext, ".ssp") == 0)
    {
        auto status = SSPResource(o->id()).save(file);
        if (status.error())
        {
            Sciwarning("SSP failed with %s", status.report().c_str());
            Scierror(999, _("%s: Unable to save \"%s\" .\n"), funname, file);
            return false;
        }
    }
    else if (strcmp(file_ext, ".xmi") == 0)
    {
        if (XMIResource(o->id()).save(file) < 0)
        {
            Scierror(999, _("%s: Unable to save \"%s\" .\n"), funname, file);
            return false;
        }
    }
    else if (strcmp(file_ext, ".dot") == 0)
    {
        // only used for debugging purpose
        auto status = SSPResource(o->id()).export_to_dot(file);
        if (status.error())
        {
            Scierror(999, _("%s: Unable to save \"%s\" .\n%s\n"), funname, file, status.report().c_str());
            return false;
        }
    }
    else
    {
        Scierror(999, _("%s: Unable to save \"%s\", unsupported extension.\n"), funname, file);
        return false;
    }

    return true;
}

==> scilab-2026.0.1.tar.bz2: scilab-2026.0.1/scilab/modules/scicos/sci_gateway/cpp/sci_scicosDiagramToScilab.cpp <==
/*
 * Scilab ( https://www.scilab.org/ ) - This file is part of Scilab
 * Copyright (C) Scilab Enterprises - 2016-2016 - Clément DAVID
 *
 * Copyright (C) 2012 - 2016 - Scilab Enterprises
 *
 * This file is hereby licensed under the terms of the GNU GPL v2.0,
 * pursuant to article 5.3.4 of the CeCILL v.2.1.
 * This file was originally licensed under the terms of the CeCILL v2.1,
 * and continues to be available under such terms.
 * For more information, see the COPYING file which you should have received
 * along with this program.
 *
 */
/*--------------------------------------------------------------------------*/
#include "Controller.hxx"
#include "view_scilab/Adapters.hxx"
#include "LoggerView.hxx"
#include "controller_helpers.hxx"
#include "XMIResource.hxx"
#include "SSPResource.hxx"

#include "types.hxx"
#include "function.hxx"
#include "string.hxx"
#include "user.hxx"
#include "int.hxx"

#include "gw_scicos.hxx"

extern "C"
{
#include "sci_malloc.h"
#include "localization.h"
#include "getFullFilename.h"
#include "Sciwarning.h"
#include "Scierror.h"
}
/*--------------------------------------------------------------------------*/
using namespace org_scilab_modules_scicos;
/*--------------------------------------------------------------------------*/
static char funname[] = "scicosDiagramToScilab";
static types::InternalType* importFile(char const* file);
static bool exportFile(int index, char const* file, types::InternalType* uid);
/*--------------------------------------------------------------------------*/
types::Function::ReturnValue sci_scicosDiagramToScilab(types::typed_list &in, int _iRetCount, types::typed_list &out)
{
    /*
     * to import some files
     * scs_m = scicosDiagramToScilab("/path/to/a/file")
     * [scs_m1, scs_m2] = scicosDiagramToScilab(["/path/to/a/file", "/path/to/files"])
     *
     * to export a file
     * scicosDiagramToScilab("/path/to/a/file", scs_m)
     * scicosDiagramToScilab(["/path/to/a/file", "/path/to/files"], scs_m1, scs_m2)
     */

    _iRetCount = std::max(1, _iRetCount);
    if (in.size() < 1)
    {
        Scierror(77, _("%s: Wrong number of input arguments: at least %d expected.\n"), funname, 1);
        return types::Function::Error;
    }

    if (!in[0]->isString())
    {
        Scierror(77, _("%s: Wrong type for input argument #%d: string expected.\n"), funname, 1);
        return types::Function::Error;
    }

    types::String* files = in[0]->getAs<types::String>();
    if (in.size() == 1 && files->getSize() == _iRetCount)
    {
        // the user asked to import
        out.resize(_iRetCount);
        for (int i = 0; i < _iRetCount; i++)
        {
            wchar_t* fullName = getFullFilenameW(files->get(i));
            char* f = wide_string_to_UTF8(fullName);
            FREE(fullName);
            out[i] = importFile(f);
            FREE(f);
            if (out[i] == nullptr)
            {
                return types::Function::Error;
            }
        }
    }
    else if (in.size() == 1 + static_cast<size_t>(files->getSize()) && _iRetCount <= 1)
    {
        // the user asked to export
        for (int i = 0; i < _iRetCount; i++)
        {
            if (!in[1 + i]->isUserType())
            {
                Scierror(77, _("%s: Wrong type for input argument #%d: ""%s"" expected.\n"), funname, i + 2, "diagram");
                return types::Function::Error;
            }
        }
        for (int i = 0; i < _iRetCount; i++)
        {
            wchar_t* fullName = getFullFilenameW(files->get(i));
            char* f = wide_string_to_UTF8(fullName);
            FREE(fullName);
            bool success = exportFile(1 + i, f, in[1 + i]);
            FREE(f);
            if (!success)
            {
                return types::Function::Error;
            }
        }
    }
    else
    {
        // report a comprehensible error message
        if (in.size() == 1)
        {
            Scierror(77, _("%s: Wrong number of output arguments: %d expected.\n"), funname, files->getSize());
        }
        else if (in.size() == 1 + static_cast<size_t>(files->getSize()))
        {
            Scierror(77, _("%s: Wrong number of output arguments: %d expected.\n"), funname, 0);
        }
        else
        {
            Scierror(77, _("%s: Wrong number of input arguments: %d expected.\n"), funname, 1);
        }
        return types::Function::Error;
    }

    return types::Function::OK;
}
/*--------------------------------------------------------------------------*/
static types::InternalType* importFile(char const* file)
{
    // create a diagram
    org_scilab_modules_scicos::Controller controller;
    ScicosID uid = controller.createObject(DIAGRAM);
    
    auto logger = get_or_allocate_logger();
    logger->log(LOG_DEBUG, [&](char* first, char* last) {
        to_chars_t io(first, last);
        io = io + "Importing file " + std::string_view(file);
        
        // DEBUG io = io + "Reset logger to " + uid;
        // DEBUG logger->setLastObject(uid);

        return io + "\n";
    });

    size_t fLen = strlen(file);
    int ret = -1;
    if (fLen < 4)
    {
        Scierror(999, _("%s: Unable to load \"%s\" .\n"), funname, file);
        controller.deleteObject(uid);
        return nullptr;
    }
    if (strcmp(&file[fLen - 4], ".ssp") == 0)
    {
        ret = SSPResource(uid).load(file);
    }
    else if (strcmp(&file[fLen - 4], ".xmi") == 0)
    {
        ret = XMIResource(uid).load(file);
    }
    else
    {
        Scierror(999, _("%s: Unable to detect \"%s\" format.\n"), funname, file);
        controller.deleteObject(uid);
        return nullptr;
    }

    if (ret < 0)
    {
        Scierror(999, _("%s: Unable to load file \"%s\" .\n"), funname, file);
        controller.deleteObject(uid);
        return nullptr;
    }

    types::InternalType* pIT = view_scilab::Adapters::instance().allocate_view(uid, DIAGRAM);
    return pIT;
}
/*--------------------------------------------------------------------------*/
static bool exportFile(int index, char const* file, types::InternalType* type)
{
    // check that the passed argument is a diagram
    const model::BaseObject* o = view_scilab::Adapters::instance().descriptor(type);
    if (o == nullptr || o->kind() != DIAGRAM)
    {
        Scierror(77, _("%s: Wrong type for input argument #%d: ""%s"" expected.\n"), funname, index, "diagram");
        return false;
    }

    size_t fLen = strlen(file);
    if (fLen < 4)
    {
        Scierror(999, _("%s: Unable to save \"%s\" .\n"), funname, file);
        return false;
    }
    
    const char* file_ext = &file[fLen - 4];
    if (strcmp(file_ext, ".ssp") == 0)
    {
        auto status = SSPResource(o->id()).save(file);
        if (status.error())
        {
            Sciwarning("SSP failed with %s", status.report().c_str());
            Scierror(999, _("%s: Unable to save \"%s\" .\n"), funname, file);
            return false;
        }
    }
    else if (strcmp(file_ext, ".xmi") == 0)
    {
        if (XMIResource(o->id()).save(file) < 0)
        {
            Scierror(999, _("%s: Unable to save \"%s\" .\n"), funname, file);
            return false;
        }
    }
    else if (strcmp(file_ext, ".dot") == 0)
    {
        // only used for debugging purpose
        auto status = SSPResource(o->id()).export_to_dot(file);
        if (status.error())
        {
            Scierror(999, _("%s: Unable to save \"%s\" .\n%s\n"), funname, file, status.report().c_str());
            return false;
        }
    }
    else
    {
        Scierror(999, _("%s: Unable to save \"%s\", unsupported extension.\n"), funname, file);
        return false;
    }

    return true;
}
